@@ -75,11 +75,19 @@ let phrase_terminator, get_phrase_terminator, set_phrase_terminator = make_varia
 let auto_run_lwt, get_auto_run_lwt, set_auto_run_lwt = make_variable true
 let auto_run_async, get_auto_run_async, set_auto_run_async = make_variable true
 let topfind_verbose, get_topfind_verbose, set_topfind_verbose = make_variable false
+let external_editor, get_external_editor, set_external_editor =
+  make_variable
+    (try
+       Sys.getenv "EDITOR"
+     with Not_found ->
+       "vi")
 
 (* Ugly hack until the action system of lambda-term is improved *)
 let end_and_accept_current_phrase : LTerm_read_line.action =
   Edit (Custom (fun () -> assert false))
 
+let set_margin_function f = UTop_private.set_margin_function f
+
 (* +-----------------------------------------------------------------+
    | Keywords                                                        |
    +-----------------------------------------------------------------+ *)
@@ -104,7 +112,7 @@ let add_keyword kwd = keywords := String_set.add kwd !keywords
 let get_message func x =
   let buffer = Buffer.create 1024 in
   let pp = Format.formatter_of_buffer buffer in
-  Format.pp_set_margin pp (S.value size).cols;
+  UTop_private.set_margin pp;
   func pp x;
   Format.pp_print_flush pp ();
   Buffer.contents buffer
@@ -112,7 +120,7 @@ let get_message func x =
 let get_ocaml_error_message exn =
   let buffer = Buffer.create 1024 in
   let pp = Format.formatter_of_buffer buffer in
-  Format.pp_set_margin pp (S.value size).cols;
+  UTop_private.set_margin pp;
   Errors.report_error pp exn;
   Format.pp_print_flush pp ();
   let str = Buffer.contents buffer in
@@ -143,10 +151,9 @@ let collect_formatters buf pps f =
   and out_spaces n = for i = 1 to n do Buffer.add_char buf ' ' done in
   let out_functions = { Format.out_string; out_flush; out_newline; out_spaces } in
   (* Replace formatter functions. *)
-  let cols = (S.value size).cols in
   List.iter
     (fun pp ->
-       Format.pp_set_margin pp cols;
+       UTop_private.set_margin pp;
        Format.pp_set_formatter_out_functions pp out_functions)
     pps;
   try
@@ -612,6 +619,11 @@ For a complete description of utop, look at the utop(1) manual page."));
             macro;
           flush stdout))
 
+let () =
+  Hashtbl.add Toploop.directive_table "pwd"
+    (Toploop.Directive_none
+       (fun () -> print_endline (Sys.getcwd ())))
+
 (* +-----------------------------------------------------------------+
    | Camlp4                                                          |
    +-----------------------------------------------------------------+ *)
