@@ -143,6 +143,11 @@ class read_phrase ~term = object(self)
   inherit [Parsetree.toplevel_phrase UTop.result * string] LTerm_read_line.engine ~history:(LTerm_history.contents UTop.history) () as super
   inherit [Parsetree.toplevel_phrase UTop.result * string] LTerm_read_line.term term as super_term
 
+  method create_temporary_file_for_external_editor =
+    Filename.temp_file "utop" ".ml"
+
+  method external_editor = UTop.get_external_editor ()
+
   val mutable return_value = None
 
   method eval =
@@ -151,7 +156,16 @@ class read_phrase ~term = object(self)
       x
     | None -> assert false
 
-  method exec = function
+  method! send_action action =
+    let action : LTerm_read_line.action =
+      if is_accept action && S.value self#mode <> LTerm_read_line.Edition then
+        Accept
+      else
+        action
+    in
+    super#send_action action
+
+  method! exec = function
     | action :: actions when S.value self#mode = LTerm_read_line.Edition &&
                              is_accept action  -> begin
         Zed_macro.add self#macro action;
@@ -178,7 +192,7 @@ class read_phrase ~term = object(self)
     | actions ->
       super_term#exec actions
 
-  method stylise last =
+  method! stylise last =
     let styled, position = super#stylise last in
 
     (* Syntax highlighting *)
@@ -199,7 +213,7 @@ class read_phrase ~term = object(self)
         (* Highlight error locations. *)
         List.iter
           (fun (start, stop) ->
-             for i = start to stop - 1 do
+             for i = max 0 start to min (Array.length styled) stop - 1 do
                let ch, style = styled.(i) in
                styled.(i) <- (ch, { style with LTerm_style.underline = Some true })
              done)
@@ -210,7 +224,7 @@ class read_phrase ~term = object(self)
 
     (styled, position)
 
-  method completion =
+  method! completion =
     let pos, words =
       UTop_complete.complete
         ~syntax:(UTop.get_syntax ())
@@ -219,7 +233,7 @@ class read_phrase ~term = object(self)
     in
     self#set_completion pos words
 
-  method show_box = S.value self#mode <> LTerm_read_line.Edition || UTop.get_show_box ()
+  method! show_box = S.value self#mode <> LTerm_read_line.Edition || UTop.get_show_box ()
 
   initializer
     (* Set the source signal for the size of the terminal. *)
@@ -243,7 +257,7 @@ let fix_string str =
     let buf = Buffer.create (len + 128) in
     if ofs > 0 then Buffer.add_substring buf str 0 ofs;
     let rec loop ofs =
-      Printf.bprintf buf "\\y%02x" (Char.code (String.unsafe_get str ofs));
+      Zed_utf8.add buf (UChar.of_char str.[ofs]);
       let ofs1 = ofs + 1 in
       let ofs2, _, _ = Zed_utf8.next_error str ofs1 in
       if ofs1 < ofs2 then
@@ -298,9 +312,12 @@ let rec map_items unwrap wrap items =
       | Outcometree.Osig_exception (name, _)
 #endif
       | Outcometree.Osig_modtype (name, _)
+#if OCAML_VERSION < (4, 03, 0)
       | Outcometree.Osig_value (name, _, _) ->
         (name, Outcometree.Orec_not)
-#if OCAML_VERSION >= (4, 03, 0)
+#else
+      | Outcometree.Osig_value { oval_name = name; _ } ->
+        (name, Outcometree.Orec_not)
       | Outcometree.Osig_ellipsis -> ("", Outcometree.Orec_not)
 #endif
     in
@@ -385,6 +402,8 @@ let with_loc loc str = {
 
 (* A rule for rewriting a toplevel expression. *)
 type rewrite_rule = {
+  type_to_rewrite : Longident.t;
+  mutable path_to_rewrite : Path.t option;
   required_values : Longident.t list;
   (* Values that must exist and be persistent for the rule to apply. *)
   rewrite : Location.t -> Parsetree.expression -> Parsetree.expression;
@@ -393,10 +412,6 @@ type rewrite_rule = {
   (* Whether the rule is enabled or not. *)
 }
 
-(* Rewrite rules, indexed by the identifier of the type
-   constructor. *)
-let rewrite_rules : (Longident.t, rewrite_rule) Hashtbl.t = Hashtbl.create 42
-
 let longident_lwt_main_run = Longident.Ldot (Longident.Lident "Lwt_main", "run")
 let longident_async_thread_safe_block_on_async_exn =
   Longident.parse "Async.Std.Thread_safe.block_on_async_exn"
@@ -422,9 +437,11 @@ let nolabel = Asttypes.Nolabel
 let nolabel = ""
 #endif
 
-let () =
+let rewrite_rules = [
   (* Rewrite Lwt.t expressions to Lwt_main.run <expr> *)
-  Hashtbl.add rewrite_rules (Longident.Ldot (Longident.Lident "Lwt", "t")) {
+  {
+    type_to_rewrite = Longident.parse "Lwt.t";
+    path_to_rewrite = None;
     required_values = [longident_lwt_main_run];
     rewrite = (fun loc e ->
 #if OCAML_VERSION < (4, 02, 0)
@@ -446,7 +463,9 @@ let () =
 
   (* Rewrite Async.Std.Defered.t expressions to
      Async.Std.Thread_safe.block_on_async_exn (fun () -> <expr>). *)
-  let rule = {
+  {
+    type_to_rewrite = Longident.parse "Async.Std.Deferred.t";
+    path_to_rewrite = None;
     required_values = [longident_async_thread_safe_block_on_async_exn];
     rewrite = (fun loc e ->
 #if OCAML_VERSION < (4, 02, 0)
@@ -468,14 +487,32 @@ let () =
 #endif
     );
     enabled = UTop.auto_run_async;
-  } in
-  Hashtbl.add rewrite_rules (Longident.parse "Async_core.Ivar.Deferred.t") rule;
-  Hashtbl.add rewrite_rules (Longident.parse "Async_kernel.Ivar.Deferred.t") rule
+  }
+]
 
-(* Returns whether the argument is a toplevel expression. *)
-let is_eval = function
-  | { Parsetree.pstr_desc = Parsetree.Pstr_eval _ } -> true
-  | _ -> false
+let rule_path rule =
+  match rule.path_to_rewrite with
+  | Some _ as x -> x
+  | None ->
+    try
+      let env = !Toploop.toplevel_env in
+      let path =
+        match Env.lookup_type rule.type_to_rewrite env with
+        | path, { Types.type_kind     = Types.Type_abstract
+                ; Types.type_private  = Asttypes.Public
+                ; Types.type_manifest = Some ty
+                } -> begin
+            match Ctype.expand_head env ty with
+            | { Types.desc = Types.Tconstr (path, _, _) } -> path
+            | _ -> path
+          end
+        | path, _ -> path
+      in
+      let opt = Some path in
+      rule.path_to_rewrite <- opt;
+      opt
+    with _ ->
+      None
 
 (* Returns whether the given path is persistent. *)
 let rec is_persistent_path = function
@@ -483,38 +520,6 @@ let rec is_persistent_path = function
   | Path.Pdot (p, _, _) -> is_persistent_path p
   | Path.Papply (_, p) -> is_persistent_path p
 
-(* Convert a path to a long identifier. *)
-let rec longident_of_path path =
-  match path with
-    | Path.Pident id ->
-      Longident.Lident (Ident.name id)
-    | Path.Pdot (path, s, _) ->
-      Longident.Ldot (longident_of_path path, s)
-    | Path.Papply (p1, p2) ->
-      Longident.Lapply (longident_of_path p1, longident_of_path p2)
-
-(* Returns the rewrite rule associated to a type, if any. *)
-let rec rule_of_type typ =
-  match typ.Types.desc with
-    | Types.Tlink typ ->
-      rule_of_type typ
-    | Types.Tconstr (path, _, _) -> begin
-      match try Some (Env.find_type path !Toploop.toplevel_env) with Not_found -> None with
-      | Some {
-        Types.type_kind = Types.Type_abstract;
-        Types.type_private = Asttypes.Public;
-        Types.type_manifest = Some typ;
-      } ->
-        rule_of_type typ
-      | _ ->
-        try
-          Some (Hashtbl.find rewrite_rules (longident_of_path path))
-        with Not_found ->
-          None
-    end
-    | _ ->
-      None
-
 (* Check that the given long identifier is present in the environment
    and is persistent. *)
 let is_persistent_in_env longident =
@@ -523,6 +528,30 @@ let is_persistent_in_env longident =
   with Not_found ->
     false
 
+let rule_matches rule path =
+  React.S.value rule.enabled &&
+  (match rule_path rule with
+   | None -> false
+   | Some path' -> Path.same path path') &&
+  List.for_all is_persistent_in_env rule.required_values
+
+(* Returns whether the argument is a toplevel expression. *)
+let is_eval = function
+  | { Parsetree.pstr_desc = Parsetree.Pstr_eval _ } -> true
+  | _ -> false
+
+(* Returns the rewrite rule associated to a type, if any. *)
+let rec rule_of_type typ =
+  match (Ctype.expand_head !Toploop.toplevel_env typ).Types.desc with
+  | Types.Tconstr (path, _, _) -> begin
+      try
+        Some (List.find (fun rule -> rule_matches rule path) rewrite_rules)
+      with _ ->
+        None
+    end
+  | _ ->
+    None
+
 #if OCAML_VERSION < (4, 02, 0)
 let rewrite_str_item pstr_item tstr_item =
   match pstr_item, tstr_item.Typedtree.str_desc with
@@ -531,11 +560,8 @@ let rewrite_str_item pstr_item tstr_item =
        Typedtree.Tstr_eval { Typedtree.exp_type = typ }) -> begin
       match rule_of_type typ with
         | Some rule ->
-          if React.S.value rule.enabled && List.for_all is_persistent_in_env rule.required_values then
-            { Parsetree.pstr_desc = Parsetree.Pstr_eval (rule.rewrite loc e);
-              Parsetree.pstr_loc = loc }
-          else
-            pstr_item
+          { Parsetree.pstr_desc = Parsetree.Pstr_eval (rule.rewrite loc e);
+            Parsetree.pstr_loc = loc }
         | None ->
           pstr_item
     end
@@ -549,11 +575,8 @@ let rewrite_str_item pstr_item tstr_item =
        Typedtree.Tstr_eval ({ Typedtree.exp_type = typ }, _)) -> begin
       match rule_of_type typ with
         | Some rule ->
-          if React.S.value rule.enabled && List.for_all is_persistent_in_env rule.required_values then
-            { Parsetree.pstr_desc = Parsetree.Pstr_eval (rule.rewrite loc e, []);
-              Parsetree.pstr_loc = loc }
-          else
-            pstr_item
+          { Parsetree.pstr_desc = Parsetree.Pstr_eval (rule.rewrite loc e, []);
+            Parsetree.pstr_loc = loc }
         | None ->
           pstr_item
     end
@@ -585,10 +608,6 @@ let rec read_phrase term =
       read_phrase term
     | exn -> Lwt.fail exn)
 
-let update_margin pp cols =
-  if Format.pp_get_margin pp () <> cols then
-    Format.pp_set_margin pp cols
-
 let print_error term msg =
   LTerm.set_style term styles.style_error >>= fun () ->
   Lwt_io.print msg >>= fun () ->
@@ -628,13 +647,12 @@ let rec loop term =
         (* Rewrite toplevel expressions. *)
         let phrase = rewrite phrase in
         (* Set the margin of standard formatters. *)
-        let cols = (LTerm.size term).cols in
-        update_margin Format.std_formatter cols;
-        update_margin Format.err_formatter cols;
+        UTop_private.set_margin Format.std_formatter;
+        UTop_private.set_margin Format.err_formatter;
         (* Formatter to get the output phrase. *)
         let buffer = Buffer.create 1024 in
         let pp = Format.formatter_of_buffer buffer in
-        Format.pp_set_margin pp (LTerm.size term).cols;
+        UTop_private.set_margin pp;
         (try
            Env.reset_cache_toplevel ();
            if !Clflags.dump_parsetree then Printast.top_phrase pp phrase;
@@ -1234,9 +1252,11 @@ let load_init_files dir =
     files
 ;;
 
-let common_init () =
+let common_init ~initial_env =
   (* Initializes toplevel environment. *)
-  Toploop.initialize_toplevel_env ();
+  (match initial_env with
+   | None -> Toploop.initialize_toplevel_env ()
+   | Some env -> Toploop.toplevel_env := env);
   (* Set the global input name. *)
   Location.input_name := UTop.input_name;
   (* Make sure SIGINT is catched while executing OCaml code. *)
@@ -1290,14 +1310,14 @@ let load_inputrc () =
       Lwt_log.error_f "error in key bindings file %S, line %d: %s" fname line msg
     | exn -> Lwt.fail exn)
 
-let main_aux () =
+let main_aux ~initial_env =
   Arg.parse args file_argument usage;
   if not (prepare ()) then exit 2;
   if !emacs_mode then begin
     UTop_private.set_ui UTop_private.Emacs;
     let module Emacs = Emacs (struct end) in
     Printf.printf "Welcome to utop version %s (using OCaml version %s)!\n\n%!" UTop.version Sys.ocaml_version;
-    common_init ();
+    common_init ~initial_env;
     Emacs.loop ()
   end else begin
     UTop_private.set_ui UTop_private.Console;
@@ -1310,7 +1330,7 @@ let main_aux () =
       (* Display a welcome message. *)
       Lwt_main.run (welcome term);
       (* Common initialization. *)
-      common_init ();
+      common_init ~initial_env;
       (* Print help message. *)
       print_string "\nType #utop_help for help about using utop.\n\n";
       flush stdout;
@@ -1329,9 +1349,9 @@ let main_aux () =
   (* Don't let the standard toplevel run... *)
   exit 0
 
-let main () =
+let main_internal ~initial_env =
   try
-    main_aux ()
+    main_aux ~initial_env
   with exn ->
     (match exn with
        | Unix.Unix_error (error, func, "") ->
@@ -1343,3 +1363,108 @@ let main () =
     Printexc.print_backtrace stderr;
     flush stderr;
     exit 2
+
+let main () = main_internal ~initial_env:None
+
+type value = V : string * _ -> value
+
+#if OCAML_VERSION < (4, 02, 0)
+
+let interact ~search_path ~unit ~loc:(fname, lnum, cnum, _) ~values =
+  failwith "UTop_main.interact is not supported on OCaml 4.01"
+
+#elif not defined ENABLE_INTERACT
+
+let interact ~search_path ~unit ~loc:(fname, lnum, cnum, _) ~values =
+  failwith "\
+UTop_main.interact wasn't enabled at compile time.
+If you installed utop through opam, just do this to enable this feature:
+
+  $ opam install ppx_tools"
+
+#else
+
+exception Found of Env.t
+
+let interact ~search_path ~unit ~loc:(fname, lnum, cnum, _) ~values =
+  let cmt_fname =
+    try
+      Misc.find_in_path_uncap search_path (unit ^ ".cmt")
+    with Not_found ->
+      Printf.ksprintf failwith "%s.cmt not found in search path!" fname
+  in
+  let cmt_infos = Cmt_format.read_cmt cmt_fname in
+  let search = object(self)
+    inherit [unit] UTop_cmt_lifter.lifter as super
+
+    method! lift_Typedtree_expression e =
+      super#lift_Typedtree_expression e;
+      match e.exp_desc with
+      | Texp_apply (_, args) -> begin
+          try
+            match
+#if OCAML_VERSION >= (4, 03, 0)
+              List.find (fun (lab, _) -> lab = Asttypes.Labelled "loc"   ) args,
+              List.find (fun (lab, _) -> lab = Asttypes.Labelled "values") args
+#else
+              List.find (fun (lab, _, _) -> lab = "loc"   ) args,
+              List.find (fun (lab, _, _) -> lab = "values") args
+#endif
+            with
+#if OCAML_VERSION >= (4, 03, 0)
+            | (Asttypes.Labelled _, Some l), (Asttypes.Labelled _, Some v) ->
+#else
+            | (_, Some l, Required), (_, Some v, Required) ->
+#endif
+              let pos = l.exp_loc.loc_start in
+              if pos.pos_fname = fname &&
+                 pos.pos_lnum = lnum   &&
+                 pos.pos_cnum - pos.pos_bol = cnum then
+                raise (Found v.exp_env)
+            | _ -> ()
+          with Not_found -> ()
+        end
+      | _ -> ()
+
+    method! lift_Types_label_description _ = ()
+    method! lift_Types_type_declaration _ = ()
+
+    method tuple _ = ()
+    method string _ = ()
+    method record _ _ = ()
+    method nativeint _ = ()
+    method list _ = ()
+    method lift_Types_Vars_t _ _ = ()
+    method lift_Types_Variance_t _ = ()
+    method lift_Types_Meths_t _ _ = ()
+    method lift_Types_Concr_t _ = ()
+    method lift_Env_t _ = ()
+    method int64 _ = ()
+    method int32 _ = ()
+    method int _ = ()
+    method constr _ _ = ()
+    method char _ = ()
+    method array _ = ()
+  end in
+  try
+    search#lift_Cmt_format_cmt_infos cmt_infos;
+    failwith "Couldn't find location in cmt file"
+  with Found env ->
+  try
+    List.iter Topdirs.dir_directory (search_path @ cmt_infos.cmt_loadpath);
+    let env = Envaux.env_of_only_summary env in
+    List.iter (fun (V (name, v)) -> Toploop.setvalue name (Obj.repr v)) values;
+    main_internal ~initial_env:(Some env)
+  with exn ->
+    Location.report_exception Format.err_formatter exn;
+    exit 2
+
+let () =
+  Location.register_error_of_exn
+    (function
+      | Envaux.Error err ->
+        Some (Location.error_of_printer_file Envaux.report_error err)
+      | _ -> None
+    )
+
+#endif
